@@ -1294,6 +1294,10 @@ impl Scatter {
 
         Ok(out.into_init())
     }
+
+    pub fn is_identity(&self) -> bool {
+        self.info().shape[..] == self.node.info().shape[..] && self.index.all_uslices().is_some()
+    }
 }
 
 #[pyfunction]
