@@ -98,7 +98,7 @@ pub struct GeneralFunctionSpec {
     #[pyo3(get)]
     pub get_shape: PyObject, // python function ([Shape])->Optional[Shape]
     #[pyo3(get)]
-    pub get_jacobians: Option<PyObject>, // python function (GeneralFunction)->List[Circuit], one circuit per input
+    pub get_jacobians: Option<PyObject>, /* python function (GeneralFunction)->List[Circuit], one circuit per input */
     #[pyo3(get)]
     pub num_non_batchable_output_dims: usize,
     #[pyo3(get)]
