@@ -136,30 +136,30 @@ pub fn einsum_pull_scatter(einsum: &Einsum) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn add_pull_scatter(add: &Add) -> Option<Scatter> {
-    let mut did_anything = false;
     let mut slices: Vec<USlice> = add
         .info()
         .shape
         .iter()
-        .map(|l| USlice { start: 0, stop: *l })
+        .map(|_l| USlice { start: 0, stop: 0 })
         .collect();
-
+    if !add
+        .nodes
+        .iter()
+        .all(|x| matches!(&***x, Circuit::Scatter(_)))
+    {
+        return None;
+    }
     for (operand, rank_difference) in add.nodes_and_rank_differences() {
-        match &**operand {
-            Circuit::Scatter(scatter) => {
-                did_anything = true;
-                for (i, (slice, l)) in
-                    zip(scatter.index.all_uslices().unwrap(), &scatter.info().shape).enumerate()
-                {
-                    if *l == add.info().shape[i + rank_difference] {
-                        slices[i + rank_difference] = slices[i + rank_difference].union(&slice);
-                    }
-                }
+        let scatter = operand.as_scatter().unwrap();
+        for (i, (slice, l)) in
+            zip(scatter.index.all_uslices().unwrap(), &scatter.info().shape).enumerate()
+        {
+            if *l == add.info().shape[i + rank_difference] {
+                slices[i + rank_difference] = slices[i + rank_difference].union(&slice);
             }
-            _ => {}
         }
     }
-    if !did_anything {
+    if zip(&slices, &add.info().shape).all(|(s, l)| s.start == 0 && s.stop == *l) {
         None
     } else {
         let new_operands = add
@@ -198,9 +198,7 @@ pub fn add_pull_scatter(add: &Add) -> Option<Scatter> {
 
 #[pyfunction]
 pub fn scatter_elim_identity(scatter: &Scatter) -> Option<CircuitRc> {
-    if scatter.info().shape[..] == scatter.node.info().shape[..]
-        && scatter.index.all_uslices().is_some()
-    {
+    if scatter.is_identity() {
         Some(scatter.node.clone())
     } else {
         None
@@ -408,5 +406,8 @@ pub fn concat_to_scatter(concat: &Concat) -> Option<Scatter> {
         concat.axis,
         concat.name_cloned(),
     );
+    if new_concat.info().numel() == concat.info().numel() {
+        return None;
+    }
     Some(Scatter::try_new(new_concat, scatter_index, concat.info().shape.clone(), None).unwrap())
 }
