@@ -212,7 +212,7 @@ impl IntoPy<PyObject> for PyUuid {
 #[derive(Debug, Clone)]
 pub struct Tensor {
     tensor: PyObject,
-    shape: Shape, // cache shape so doesn't have to be recomputed on reconstruct etc (not uber efficient I think)
+    shape: Shape, /* cache shape so doesn't have to be recomputed on reconstruct etc (not uber efficient I think) */
     hash: Option<HashBytes>,
 }
 
