@@ -20,7 +20,7 @@ use uuid::uuid;
 /// OpSize is a memory optimization of Option<usize> that stores a 63-bit int and one "is this none" bit
 #[derive(Copy, Clone, PartialEq, Eq)]
 pub struct OpSize(pub u64);
-pub type OpShape = Vec<OpSize>;
+pub type OpShape = Sv<[OpSize; 6]>;
 
 impl OpSize {
     // sometimes into/from can't do type interference, so we have aliases
@@ -152,7 +152,7 @@ impl RearrangeSpec {
         RearrangeSpec {
             input_ints: (0..rank).map(|i| sv![i as u8]).collect(),
             output_ints: sv![(0..rank as u8).collect()],
-            int_sizes: vec![OpSize::from(None); rank],
+            int_sizes: sv![OpSize::from(None); rank],
         }
     }
 
@@ -172,7 +172,7 @@ impl RearrangeSpec {
         RearrangeSpec {
             input_ints: (0..rank).map(|i| sv![i as u8]).collect(),
             output_ints: (0..rank).map(|i| sv![i as u8]).collect(),
-            int_sizes: vec![OpSize::from(None); rank],
+            int_sizes: sv![OpSize::from(None); rank],
         }
     }
     pub fn to_py_rearrange_spec(&self, input_shape: Shape) -> PyObject {
@@ -387,27 +387,23 @@ impl RearrangeSpec {
     /// # Examples
     /// 1. if special_case_ones, remove all indices of size 1
     /// ```
-    /// # use rust_circuit::rearrange_spec::{OpSize, RearrangeSpec};
+    /// # use rust_circuit::{sv, rearrange_spec::{OpSize, RearrangeSpec}};
     /// let spec = RearrangeSpec::new(
-    ///     vec![vec![0, 1], vec![2, 3]],
-    ///     vec![vec![3, 2, 1, 0]],
-    ///     vec![OpSize(3), OpSize(1), OpSize(1), OpSize(1)],
-    /// );
-    /// let canonicalized_spec = RearrangeSpec::new(
-    ///     vec![vec![0], vec![]],
-    ///     vec![vec![0]],
-    ///     vec![OpSize(3)],
+    ///     sv![sv![0, 1], sv![2, 3]],
+    ///     sv![sv![3, 2, 1, 0]],
+    ///     sv![OpSize(3), OpSize(1), OpSize(1), OpSize(1)],
     /// );
+    /// let canonicalized_spec = RearrangeSpec::new(sv![sv![0], sv![]], sv![sv![0]], sv![OpSize(3)]);
     /// assert_eq!(spec.canonicalize(true), canonicalized_spec);
     /// ```
     /// 2. Merge all sequences of indices that always appear together in the same order inside parentheses
     /// 3. Renumber the int indices sequentially based on when they first appear, inputs first then outputs
     /// ```
-    /// # use rust_circuit::rearrange_spec::{OpSize, RearrangeSpec};
+    /// # use rust_circuit::{sv, rearrange_spec::{OpSize, RearrangeSpec}};
     /// let spec = RearrangeSpec::new(
-    ///     vec![vec![0], vec![1, 2], vec![3, 4, 5]],
-    ///     vec![vec![1, 2], vec![5], vec![3, 4], vec![0]],
-    ///     vec![
+    ///     sv![sv![0], sv![1, 2], sv![3, 4, 5]],
+    ///     sv![sv![1, 2], sv![5], sv![3, 4], sv![0]],
+    ///     sv![
     ///         OpSize(2),
     ///         OpSize(2),
     ///         OpSize(2),
@@ -417,9 +413,9 @@ impl RearrangeSpec {
     ///     ],
     /// );
     /// let canonicalized_spec = RearrangeSpec::new(
-    ///     vec![vec![0], vec![1], vec![2, 3]],
-    ///     vec![vec![1], vec![3], vec![2], vec![0]],
-    ///     vec![OpSize(2), OpSize(4), OpSize(4), OpSize(2)],
+    ///     sv![sv![0], sv![1], sv![2, 3]],
+    ///     sv![sv![1], sv![3], sv![2], sv![0]],
+    ///     sv![OpSize(2), OpSize(4), OpSize(4), OpSize(2)],
     /// );
     /// assert_eq!(spec.canonicalize(true), canonicalized_spec);
     /// ```
@@ -633,7 +629,7 @@ impl RearrangeSpec {
     pub fn conform_to_input_shape(
         &self,
         shape: &Shape,
-        coerce: bool, // if false, only change None sizes, if true, change others if unique solution
+        coerce: bool, /* if false, only change None sizes, if true, change others if unique solution */
     ) -> Result<RearrangeSpec, TensorInvariantError> {
         if shape.len() != self.input_ints.len() {
             return Err(TensorInvariantError::RearrangeInputNotConformable {
@@ -769,17 +765,13 @@ impl RearrangeSpec {
     ///
     /// # Examples
     /// ```
-    /// # use rust_circuit::rearrange_spec::{OpSize, RearrangeSpec};
+    /// # use rust_circuit::{sv, rearrange_spec::{OpSize, RearrangeSpec}};
     /// let inner = RearrangeSpec::new(
     ///     sv![sv![0], sv![1]],
     ///     sv![sv![0, 1]],
     ///     sv![OpSize(2), OpSize(3)],
     /// );
-    /// let outer = RearrangeSpec::new(
-    ///     sv![sv![0]],
-    ///     sv![sv![0], sv![1]],
-    ///     sv![OpSize(6), OpSize(4)],
-    /// );
+    /// let outer = RearrangeSpec::new(sv![sv![0]], sv![sv![0], sv![1]], sv![OpSize(6), OpSize(4)]);
     /// let fused = RearrangeSpec::new(
     ///     sv![sv![0], sv![1]],
     ///     sv![sv![0, 1], sv![2]],
