@@ -1,4 +1,5 @@
-use crate::pyo3_prelude::*;
+use crate::hashmaps::FxHashMap as HashMap;
+use crate::smallvec::Sv;
 use crate::{
     circuit::{CircuitConstructionError, HashBytes},
     filter_by_variant,
@@ -6,6 +7,7 @@ use crate::{
     pycall,
     rearrange_spec::RearrangeSpec,
 };
+use crate::{pyo3_prelude::*, sv};
 use pyo3::types::IntoPyDict;
 use pyo3::{
     exceptions, ffi, pyclass, pymethods,
@@ -14,17 +16,17 @@ use pyo3::{
 };
 use std::{
     cmp::{max, min},
-    collections::HashMap,
     fmt::{self, Display},
     iter::zip,
 };
+
 use thiserror::Error;
 use uuid::uuid;
 
 use super::py_types::ExtraPySelfOps;
 
 // make me small vec for perf
-pub type Shape = Vec<usize>; // TODO: maybe this should be struct which preserves invariant that dim shapes >= 0?
+pub type Shape = Sv<[usize; 6]>; // TODO: maybe this should be struct which preserves invariant that dim shapes >= 0?
 
 #[derive(Error, Debug, Clone)]
 pub enum TensorInvariantError {
@@ -47,11 +49,11 @@ impl From<TensorInvariantError> for PyErr {
 }
 pub fn broadcast_shapes(shapes: &Vec<Shape>) -> Result<Shape, TensorInvariantError> {
     if shapes.is_empty() {
-        return Ok(vec![]);
+        return Ok(sv![]);
     }
     let ranks: Vec<usize> = shapes.iter().map(|x| x.len()).collect();
     let out_rank = *ranks.iter().max().unwrap();
-    let mut result = vec![1; out_rank];
+    let mut result: Shape = sv![1; out_rank];
     for axis_from_end in 1..out_rank + 1 {
         for (i, shape) in shapes.iter().enumerate() {
             let axis_for_shape = ranks[i].checked_sub(axis_from_end);
@@ -87,26 +89,25 @@ pub struct Slice {
 impl Slice {
     pub fn to_unsigned_loc(x: i64, l: usize) -> usize {
         if x < 0 {
-            l.checked_sub((-x) as usize).unwrap()
+            l.saturating_sub((-x) as usize)
         } else {
-            assert!(x <= l as i64);
-            x as usize
+            l.min(x as usize)
         }
     }
 
-    pub fn start_u(self, l: usize) -> usize {
+    fn start_u(self, l: usize) -> usize {
         self.start.map(|x| Self::to_unsigned_loc(x, l)).unwrap_or(0)
     }
 
-    pub fn stop_u(self, l: usize) -> usize {
+    fn stop_u(self, l: usize) -> usize {
         self.stop.map(|x| Self::to_unsigned_loc(x, l)).unwrap_or(l)
     }
 
-    pub fn size(self, l: usize) -> usize {
+    fn size(self, l: usize) -> usize {
         self.stop_u(l).saturating_sub(self.start_u(l))
     }
 
-    pub fn canonicalize(self, l: usize) -> Self {
+    fn canonicalize(self, l: usize) -> Self {
         Self {
             start: Some(self.start_u(l) as i64),
             stop: Some(self.stop_u(l) as i64),
@@ -128,7 +129,7 @@ impl Slice {
         }
     }
 
-    pub fn is_identity(self, l: usize) -> bool {
+    fn is_identity(self, l: usize) -> bool {
         self.start_u(l) == 0 && self.stop_u(l) == l
     }
 }
